@@ -141,7 +141,7 @@ class Session:
         method: str,
         url: str,
         params: Optional[dict] = None,
-        data: Optional[Union[Dict[str, str], BytesIO, bytes]] = None,
+        data: Optional[Union[Dict[str, str], str, BytesIO, bytes]] = None,
         json: Optional[dict] = None,
         headers: Optional[HeaderTypes] = None,
         cookies: Optional[CookieTypes] = None,
@@ -171,6 +171,8 @@ class Session:
         # data/body/json
         if isinstance(data, dict):
             body = urlencode(data).encode()
+        elif isinstance(data, str):
+            body = data.encode()
         elif isinstance(data, BytesIO):
             body = data.read()
         elif isinstance(data, bytes):
@@ -178,7 +180,7 @@ class Session:
         elif data is None:
             body = b""
         else:
-            raise TypeError("data must be dict, BytesIO or bytes")
+            raise TypeError("data must be dict, str, BytesIO or bytes")
         if json:
             body = dumps(json).encode()
         if body:
@@ -247,16 +249,18 @@ class Session:
             proxies = {**self.proxies, **(proxies or {})}
         if proxies:
             if url.startswith("http://"):
-                c.setopt(CurlOpt.PROXY, proxies["http"])
+                if proxies["http"] is not None:
+                    c.setopt(CurlOpt.PROXY, proxies["http"])
             elif url.startswith("https://"):
-                if proxies["https"].startswith("https://"):
-                    raise RequestsError(
-                        "You are using http proxy WRONG, the prefix should be 'http://' not 'https://', see: https://github.com/yifeikong/curl_cffi/issues/6"
-                    )
-                c.setopt(CurlOpt.PROXY, proxies["https"])
-                # for http proxy, need to tell curl to enable tunneling
-                if not proxies["https"].startswith("socks"):
-                    c.setopt(CurlOpt.HTTPPROXYTUNNEL, 1)
+                if proxies["https"] is not None:
+                    if proxies["https"].startswith("https://"):
+                        raise RequestsError(
+                            "You are using http proxy WRONG, the prefix should be 'http://' not 'https://', see: https://github.com/yifeikong/curl_cffi/issues/6"
+                        )
+                    c.setopt(CurlOpt.PROXY, proxies["https"])
+                    # for http proxy, need to tell curl to enable tunneling
+                    if not proxies["https"].startswith("socks"):
+                        c.setopt(CurlOpt.HTTPPROXYTUNNEL, 1)
 
         # verify
         if verify is False or not self.verify and verify is None:
@@ -279,9 +283,9 @@ class Session:
             c.impersonate(impersonate)
 
         buffer = BytesIO()
-        c.setopt(CurlOpt.WRITEFUNCTION, buffer.write)
+        c.setopt(CurlOpt.WRITEDATA, buffer)
         header_buffer = BytesIO()
-        c.setopt(CurlOpt.HEADERFUNCTION, header_buffer.write)
+        c.setopt(CurlOpt.HEADERDATA, header_buffer)
 
         try:
             c.perform()
